#include "render.h"
#include "QTime"
#include "qrect.h"
#include "qfont.h"
#include "qfile.h"
#include "qdatastream.h"
#include "qmessagebox.h"
#include "QtCore"
#include "qdebug.h"
#include "charautomation.h"

#ifndef GL_MULTISAMPLE
#define GL_MULTISAMPLE  0x809D
#endif




GLWidget::GLWidget(QWidget *parent)
    : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{
        QTime midnight(0, 0, 0);
         qsrand(midnight.secsTo(QTime::currentTime()));

         setAutoFillBackground(false);
         startTimer(30);
         setWindowTitle(tr("Mappy Engine"));


         //Character
         Player = new Character("NPC03",4);

         Player->PositionX = width() / 2;
         Player->PositionY = height() / 2;
/*
         NPC00 = new Character("NPC00",4);
         NPC00->PositionX = 192; NPC00->PositionY = 128;

         NPC01 = new Character("NPC01",4);
         NPC02 = new Character("NPC03",4);
         NPC02->PositionX = 64; NPC02->PositionY = 192;
*/

         //MappySaver m;
         logo = QPixmap(":/images/data/logo_transparent.png");





}


GLWidget::~GLWidget()
{
}



void GLWidget::mouseMoveEvent(QMouseEvent *event)
 {

}

void GLWidget::mousePressEvent(QMouseEvent *event)
{
    mPressLoc = QPoint(event->x() / 32 * 32 ,event->y() / 32 * 32);
}


void GLWidget::paintEvent(QPaintEvent *event)
 {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QFont font = painter.font();
    font.setPointSize(font.pointSize() * 2);

    painter.setFont(font);


    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

     makeCurrent();
     glMatrixMode(GL_MODELVIEW);
     glPushMatrix();

     //QColor qtGreen = QColor::fromCmykF(0.40, 0.0, 1.0, 0.0);

     QColor violet = QColor::fromRgb(153,204,255);
     qglClearColor(violet.dark());

     glShadeModel(GL_SMOOTH);
     glEnable(GL_DEPTH_TEST);
     glEnable(GL_CULL_FACE);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
     glEnable(GL_MULTISAMPLE);
     static GLfloat lightPosition[4] = { 0.5, 5.0, 7.0, 1.0 };
     glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);


         glShadeModel(GL_FLAT);
         glDisable(GL_CULL_FACE);
         glDisable(GL_DEPTH_TEST);
         glDisable(GL_LIGHTING);

         glMatrixMode(GL_MODELVIEW);
         glPopMatrix();


         //Scrolling
         painter.translate(0,(this->height() /2) - Player->PositionY);
         //painter.translate(camX,camY);


          //GL Overlay
         painter.drawPixmap(QPoint(mappa.Tiles[0].x,mappa.Tiles[0].y),Tileset,QRect(0,0,32,32));

          x++;


          //Draw Tiles
          for(int i = 0; i < mappa.Tiles.count(); i++)
          {
              painter.drawPixmap(mappa.Tiles[i].x,mappa.Tiles[i].y,Tileset,mappa.Tiles[i].rect.x(),mappa.Tiles[i].rect.y(),mappa.Tiles[i].rect.width()- mappa.Tiles[i].rect.x() + 32,mappa.Tiles[i].rect.height() - mappa.Tiles[i].rect.y() + 32);
          }





       //   NPC02->Draw(painter);
        //  NPC00->Draw(painter);
          Player->Draw(painter);


          painter.drawStaticText(5,5,QString("%1 x %2").arg(mPressLoc.x()).arg(mPressLoc.y()));


          painter.end();


         //swapBuffers();
        // update();  //Soo Fast! disabled...
        // updateGL();



         Player->Update();
       //  NPC00->Update();
       //  NPC02->Update();





}


void GLWidget::initializeGL()
 {
    glEnable(GL_MULTISAMPLE);


     //logo = new QtLogo(this);
    // logo->setColor(qtGreen.dark());

    static const char *vertexShaderSource =
        "attribute highp vec4 posAttr;\n"
        "attribute lowp vec4 colAttr;\n"
        "varying lowp vec4 col;\n"
        "uniform highp mat4 matrix;\n"
        "void main() {\n"
        "   col = colAttr;\n"
        "   gl_Position = matrix * posAttr;\n"
        "}\n";

    static const char *fragmentShaderSource =
        "varying lowp vec4 col;\n"
        "void main() {\n"
        "   gl_FragColor = col;\n"
        "}\n";


 }


//! Initialization
void GLWidget::glInit()
{
    //Loading First Asset

    QString path = QDir::currentPath();
    path.append(QString("/data/Map/%1").arg(levelname));

    //Recupero il file mappa
    QFile file(path);
    if(!file.open(QIODevice::ReadOnly))
    {
        QMessageBox::information(this,tr("Errore"),tr("File not found"),QMessageBox::Ok);

    }else
    {
        QDataStream in(&file);
        in.setVersion(QDataStream::Qt_4_8);

        //Load all pice of tiles
        in >> mappa.Tiles;
        file.flush();


        QByteArray array;
        in >> array;

        QString tilename = QString::fromLocal8Bit(array);

        QStringList sp = tilename.split('/');

        QString filename = sp[sp.count()-1];



        try
        {
            Tileset.load(QString("data/Tileset/%1").arg(filename));

        } catch(int d)
        {

            Tileset.load(tilename);
        }


    }







}

void GLWidget::resizeGL(int width, int height)
 {
     setupViewport(width, height);
 }


void GLWidget::timerEvent(QTimerEvent *)
{
    update();
}

void GLWidget::setupViewport(int width, int height)
{

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();


#ifdef QT_OPENGL_ES
    glOrthof(0,render_width,0,render_height,-1,1);
#else
    glOrtho(0,render_width,0,render_height,-1,1);
#endif

    glViewport(0,0,render_width,render_height);

}


void GLWidget::LoadTexture(QString filename)
{

}


void GLWidget::keyPressEvent(QKeyEvent *e)
{



    if(e->key() == Qt::Key_Down)
    {
       Player->MoveDown();
    }

    if(e->key() == Qt::Key_Up)
    {
        Player->MoveUp();
    }

    if(e->key() == Qt::Key_Right)
    {
        Player->MoveRight();
    }

    if(e->key() == Qt::Key_Left)
    {
        Player->MoveLeft();
    }

    if(e->key() == Qt::Key_W)
    {
        NPC00->MoveDown();
        NPC02->MoveRight();
    }
}
